@@ -12,9 +12,13 @@ import os
 import time
 
 next_send_time = {}
+count_by_chat_id = {}
 def waitSend(chat_id, media_group_size):
-	wait = next_send_time.get(chat_id, 0) + media_group_size ** 2 + media_group_size * 10 - time.time()
-	next_send_time[chat_id] = time.time() + media_group_size ** 2 + media_group_size * 10
+	if next_send_time.get(chat_id, 0) < time.time() - 10 * 60:
+		count_by_chat_id[chat_id] = 0
+	count_by_chat_id[chat_id] = count_by_chat_id.get(chat_id, 0) + media_group_size + 2
+	wait = next_send_time.get(chat_id, 0) + media_group_size ** 2 + media_group_size * 10 - time.time() + media_group_size * 10 + min(30, count_by_chat_id[chat_id]) - 15
+	next_send_time[chat_id] = time.time() + media_group_size ** 2
 	if wait > 0:
 		time.sleep(wait)
 	
@@ -78,7 +82,7 @@ def getImage(img):
 	cached_url.get(img, force_cache=True, mode='b')
 	return cached_url.getFilePath(img)
 
-def send_v2(chat, result, rotate=0, send_all=False, no_cut=False, size_factor = None):
+def send_v2_imp(chat, result, rotate=0, send_all=False, no_cut=False, size_factor = None, rotate_detail = {}, start_page = 0):
 	# todo: cached_url may want to make sure the video fn ends with mp4...	
 	if result.video:
 		try:
@@ -87,7 +91,7 @@ def send_v2(chat, result, rotate=0, send_all=False, no_cut=False, size_factor =
 			if not result.imgs:
 				raise e
 
-	img_limit = 100 if send_all else 10
+	img_limit = start_page * 10 + (100 if send_all else 10)
 	if no_cut:
 		imgs = [getImage(img) for img in result.imgs]
 	else:
@@ -97,6 +101,9 @@ def send_v2(chat, result, rotate=0, send_all=False, no_cut=False, size_factor =
 		imgs = pic_cut.getCutImages(result.imgs, img_limit, **args)	
 	imgs = [x for x in imgs if properSize(x)]
 	[imgRotate(x, rotate) for x in imgs]
+	for index, rotate_single in rotate_detail.items():
+		imgRotate(imgs[index], rotate_single)
+	imgs = imgs[start_page * 10:]
 	
 	if imgs:
 		return_result = []
@@ -112,6 +119,15 @@ def send_v2(chat, result, rotate=0, send_all=False, no_cut=False, size_factor =
 			parse_mode=result.getParseMode(), timeout = 20*60, 
 			disable_web_page_preview = (not isUrl(result.cap)))]
 
+def send_v2(chat, result, rotate=0, send_all=False, no_cut=False, size_factor = None, rotate_detail = {}, start_page=0):
+	try:
+		return send_v2_imp(chat, result, rotate=rotate, send_all=send_all, no_cut=no_cut, size_factor = size_factor, rotate_detail = rotate_detail, start_page=start_page)
+	except Exception as e:
+		if "Can't parse entities" in str(e):
+			result.no_parse = True
+			return send_v2_imp(chat, result, rotate=rotate, send_all=send_all, no_cut=no_cut, size_factor = size_factor, rotate_detail = rotate_detail, start_page=start_page)
+		raise e
+
 def send(chat, url, result, rotate=0, send_all=False):
 	result.url = url
 	send_v2(chat, result, rotate=rotate, send_all=send_all)
\ No newline at end of file
